"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _utilsConfig = require("../../../utils/config");

var _utilsConfig2 = _interopRequireDefault(_utilsConfig);

var getCommonTasks = function getCommonTasks(buildName, siteConfig, builtInPlugins) {

    /*
        This gives us all extensions which are build specific.
        eg ["~client", "~dev"]
    */
    var browserBuildFileSuffixes = Object.keys(siteConfig.builds).map(function (key) {
        return siteConfig.builds[key]["browser-build-file-suffix"];
    }).filter(function (suffix) {
        return typeof suffix !== "undefined" && suffix !== null;
    });

    var excludedBuildSpecificFilePatterns = browserBuildFileSuffixes.map(function (s) {
        return s + ".(js|json)$";
    });

    /*
        Read builds <buildname>.tasks.<name>.
        If not found, check builds.common
    */
    var getBuildConfigReader = function getBuildConfigReader(buildName) {
        var buildReader = _utilsConfig2["default"].getReader(siteConfig, ["builds", buildName]);
        var commonReader = _utilsConfig2["default"].getReader(siteConfig, ["builds", "common"]);
        return function (readPath, defaultValue) {
            return buildReader(readPath, commonReader(readPath, defaultValue));
        };
    };

    /*
        Read tasks <buildname>.tasks.<name>.
        If not found, check builds.common.tasks.<name>
    */
    var getTaskConfigReader = function getTaskConfigReader(buildName, name) {
        var taskReader = _utilsConfig2["default"].getReader(siteConfig, ["builds", buildName, "tasks", name]);
        var commonReader = _utilsConfig2["default"].getReader(siteConfig, ["builds", "common", "tasks", name]);
        return function (readPath, defaultValue) {
            return taskReader(readPath, commonReader(readPath, defaultValue));
        };
    };

    var getTranspileServerTask = function getTranspileServerTask(options) {
        var dependencies = options.dependencies;
        var destination = siteConfig.destination;
        var extensions = siteConfig["js-extensions"];
        var excludedPatterns = siteConfig["excluded-patterns"].map(function (p) {
            return { exclude: p.exclude, regex: new RegExp(p.regex) };
        });

        var taskConfigReader = getTaskConfigReader(buildName, options.name);

        var vendorDirs = taskConfigReader(["vendor-dirs"], ["vendor"]);
        var excludedDirectories = [siteConfig.destination].concat(vendorDirs).concat(siteConfig["excluded-dirs"]);
        var blacklist = ["regenerator", "es6.constants", "es6.blockScoping"];

        return {
            name: options.name, //babel transpile server files, blacklist (regenerator)
            plugin: builtInPlugins.babel,
            options: {
                destination: destination,
                extensions: extensions,
                excludedDirectories: excludedDirectories,
                excludedPatterns: excludedPatterns,
                excludedWatchPatterns: excludedBuildSpecificFilePatterns,
                blacklist: blacklist,
                dependencies: dependencies
            }
        };
    };

    var getLessTask = function getLessTask(options) {
        var dependencies = options.dependencies;
        var source = siteConfig.source;
        var destination = siteConfig.destination;

        var taskConfigReader = getTaskConfigReader(buildName, options.name);

        var files = taskConfigReader(["files"], ["css/main.less"]);

        return {
            name: options.name, //compile less files
            plugin: builtInPlugins.less,
            options: {
                source: source,
                destination: destination,
                files: files,
                dependencies: dependencies
            }
        };
    };

    var getCopyStaticFilesTask = function getCopyStaticFilesTask(options) {
        var dependencies = options.dependencies;
        var destination = options.destination;
        var extensions = options.extensions || ["*.*"];
        var excludedDirectories = [siteConfig.destination].concat(siteConfig["excluded-dirs"]);
        var excludedPatterns = siteConfig["excluded-patterns"];

        var taskConfigReader = getTaskConfigReader(buildName, options.name);

        var excludedExtensions = taskConfigReader(["excluded-extensions"], ["less"]);
        var changeExtensions = taskConfigReader(["change-extensions"], [{ to: "js", from: ["jsx"] }]);

        return {
            name: options.name,
            plugin: builtInPlugins["copy-static-files"],
            options: {
                destination: destination,
                extensions: extensions,
                excludedDirectories: excludedDirectories,
                excludedPatterns: excludedPatterns,
                excludedExtensions: excludedExtensions,
                excludedWatchPatterns: excludedBuildSpecificFilePatterns,
                changeExtensions: changeExtensions,
                dependencies: dependencies
            }
        };
    };

    var getWriteConfigTask = function getWriteConfigTask(options) {
        var dependencies = options.dependencies;
        var destination = _path2["default"].join(siteConfig.destination);

        var taskConfigReader = getTaskConfigReader(buildName, options.name);

        var filename = taskConfigReader(["filename"], "config.json");

        return {
            name: options.name,
            plugin: builtInPlugins["write-config"],
            options: {
                destination: destination,
                filename: filename,
                config: siteConfig,
                dependencies: dependencies
            }
        };
    };

    var getBuildClientTask = function getBuildClientTask(options) {
        var dependencies = options.dependencies;
        var excludedDirectories = siteConfig["excluded-dirs"].concat(siteConfig["custom-builds-dir"], siteConfig["custom-tasks-dir"]);
        var excludedPatterns = siteConfig["excluded-patterns"];
        var changeExtensions = siteConfig["change-extensions"];
        var jsExtensions = siteConfig["js-extensions"].concat("json");

        var buildConfigReader = getBuildConfigReader(buildName);
        var taskConfigReader = getTaskConfigReader(buildName, options.name);

        var vendorDirs = taskConfigReader(["vendor-dirs"], ["vendor"]);
        var clientBuildDirectory = options.clientBuildDirectory || buildConfigReader(["client-build-dir"], "js");
        var appEntryPoint = taskConfigReader(["entry-point"], "app.js");
        var bundleName = taskConfigReader(["bundle-name"], "app.bundle.js");
        var globalModules = options.globalModules || taskConfigReader(["global-modules"], []);
        var excludedModules = options.excludedModules || taskConfigReader(["excluded-modules"], []);
        var browserBuildFileSuffix = options.browserBuildFileSuffix;
        var browserReplacedFileSuffix = options.browserReplacedFileSuffix;
        var blacklist = taskConfigReader(["es6-transpile", "blacklist"], []);

        var excludedWatchPatterns = taskConfigReader(["excluded-watch-patterns"], [browserBuildFileSuffixes.filter(function (s) {
            return s !== browserBuildFileSuffix;
        })]).map(function (e) {
            return new RegExp(e + ".(js|json)$");
        });

        return {
            name: options.name,
            plugin: builtInPlugins["build-browser-app"],
            options: {
                source: siteConfig.source,
                destination: siteConfig.destination,
                clientBuildDirectory: clientBuildDirectory,
                appEntryPoint: appEntryPoint,
                bundleName: bundleName,
                jsExtensions: jsExtensions,
                changeExtensions: changeExtensions,
                debug: options.debug,
                globalModules: globalModules,
                excludedModules: excludedModules,
                excludedDirectories: vendorDirs.concat(excludedDirectories),
                excludedPatterns: excludedPatterns,
                browserBuildFileSuffix: browserBuildFileSuffix,
                browserReplacedFileSuffix: browserReplacedFileSuffix,
                excludedWatchPatterns: excludedWatchPatterns,
                blacklist: blacklist,
                dependencies: dependencies
            }
        };
    };

    var getLoadDataTask = function getLoadDataTask(options) {
        var dependencies = options.dependencies;
        var scavengeCollectionDependencies = options.scavengeCollectionDependencies;
        var collectionLoaderDependencies = options.collectionLoaderDependencies;
        var collections = siteConfig.collections;
        var collectionRootDirectory = siteConfig["collections-root-dir"];
        var dataDirectories = siteConfig["data-dirs"];
        var scavengeCollection = siteConfig["scavenge-collection"] || "posts";

        var buildConfigReader = getBuildConfigReader(buildName);
        var taskConfigReader = getTaskConfigReader(buildName, options.name);

        var vendorDirs = taskConfigReader(["vendor-dirs"], ["vendor"]);
        var excludedDirectories = siteConfig["excluded-dirs"].concat([siteConfig["custom-builds-dir"], siteConfig["custom-tasks-dir"]]).concat(vendorDirs);

        var excludedFiles = taskConfigReader(["excluded-files"], ["config.yml", "config.yaml", "config.json", "package.json", "README.md"]);
        var markdownExtensions = taskConfigReader(["markdown-extensions"], ["md", "markdown"]);

        return {
            name: options.name,
            plugin: builtInPlugins["load-data"],
            options: {
                data: options.data,
                collections: collections,
                collectionRootDirectory: collectionRootDirectory,
                dataDirectories: dataDirectories,
                scavengeCollection: scavengeCollection,
                excludedDirectories: excludedDirectories,
                excludedFiles: excludedFiles,
                markdownExtensions: markdownExtensions,
                dependencies: dependencies,
                scavengeCollectionDependencies: scavengeCollectionDependencies,
                collectionLoaderDependencies: collectionLoaderDependencies
            }
        };
    };

    return { getTranspileServerTask: getTranspileServerTask, getLessTask: getLessTask, getCopyStaticFilesTask: getCopyStaticFilesTask, getWriteConfigTask: getWriteConfigTask, getBuildClientTask: getBuildClientTask, getLoadDataTask: getLoadDataTask };
};

exports["default"] = getCommonTasks;
module.exports = exports["default"];