"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _optimist = require("optimist");

var _optimist2 = _interopRequireDefault(_optimist);

var argv = _optimist2["default"].argv;

var isEmpty = function isEmpty(val) {
    return typeof val === "undefined" || val === null;
};

/*
    Tries to read a.b.c.d when passed a, ["b", "c", "d"]
    If anything is undefined, returns defaultVal
*/
var tryRead = function tryRead(obj, path, defaultVal) {
    var currentVal = obj;
    for (var i = 0; i < path.length; i++) {
        var p = path[i];
        if (typeof currentVal[p] !== "undefined") {
            currentVal = currentVal[p];
        } else {
            if (typeof defaultVal !== "undefined") {
                return defaultVal;
            } else {
                throw new Error(path.join(".") + " is mandatory in " + JSON.stringify(obj));
            }
        }
    }
    return currentVal;
};

var getReader = function getReader(obj, defaultPath) {
    return function (remainingPath, defaultValue) {
        return tryRead(obj, defaultPath.concat(remainingPath), defaultValue);
    };
};

var getProperty = function getProperty(config, fullyQualifiedProperty) {
    //props are like "a.b.c"; we need to find config.a.b.c
    //propParent will be config.a.b, in this case.
    var propArray = fullyQualifiedProperty.split(".");
    var propertyName = propArray.slice(propArray.length - 1)[0];
    var propParents = propArray.slice(0, propArray.length - 1);

    //Make sure a.b exists in config
    var parentProperty = config;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = propParents[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _parent = _step.value;

            if (isEmpty(parentProperty[_parent])) {
                parentProperty[_parent] = {};
            }
            parentProperty = parentProperty[_parent];
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
                _iterator["return"]();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return { propertyName: propertyName, parentProperty: parentProperty };
};

/*
    getValueSetter() returns a valueSetter function.
    valueSetter initializes config properties with
        a) command-line params, if specified
        b) default values, if property is currently empty
*/
var getValueSetter = function getValueSetter(config) {
    return function (fullyQualifiedProperty, defaultValue) {
        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

        var _getProperty = getProperty(config, fullyQualifiedProperty);

        var propertyName = _getProperty.propertyName;
        var parentProperty = _getProperty.parentProperty;

        if (isEmpty(parentProperty[propertyName])) {
            parentProperty[propertyName] = defaultValue;
        } else {
            if (options.replace) {
                parentProperty[propertyName] = defaultValue;
            }
        }
    };
};

var commandLineSetter = function commandLineSetter(config) {
    for (var cmd in argv) {
        var isArray = false;
        if (cmd !== "_" && !/^\$/.test(cmd) && ["s", "d", "source", "destination"].indexOf(cmd) === -1) {
            if (/\[\]$/.test(cmd)) {
                cmd = cmd.replace(/\[\]$/, "");
                isArray = true;
            }

            var _getProperty2 = getProperty(config, cmd);

            var propertyName = _getProperty2.propertyName;
            var parentProperty = _getProperty2.parentProperty;

            var commandLineValue = argv[cmd];
            if (!isEmpty(commandLineValue)) {
                if (isEmpty(parentProperty[propertyName])) {
                    parentProperty[propertyName] = isArray ? [].concat(commandLineValue) : commandLineValue;
                } else {
                    //If current prop is not empty, but is an array, replace if
                    // flag is set. Otherwise concat.
                    if (parentProperty[propertyName] instanceof Array) {
                        if (argv[fullyQualifiedProperty + "-replace"]) {
                            parentProperty[propertyName] = [].concat(commandLineValue);
                        } else {
                            parentProperty[propertyName] = parentProperty[propertyName].concat(commandLineValue);
                        }
                    } else {
                        //If current prop is not an array, replace anyway.
                        parentProperty[propertyName] = isArray ? [].concat(commandLineValue) : commandLineValue;
                    }
                }
            }
        }
    }
};

/*
    example of obj:
    {
        prop1: false,
        prop2: "hello",
        prop3: { value: ["A"], options: { replace: true } }
        prop4: {
            propInner1: 100
        }
    }

    returns [
        ["prop1", false],
        ["prop2", "hello"],
        ["prop3", "A", { replace: true}],
        ["prop4.propInner", 1000]
    ]
}
*/
var getFullyQualifiedProperties = function getFullyQualifiedProperties(obj) {
    var prefixes = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
    var acc = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

    for (var key in obj) {
        var val = obj[key];
        var fullNameArray = prefixes.concat(key);
        if (!(val instanceof Array) && val !== null && typeof val === "object" && typeof val.value === "undefined") {
            acc.push([fullNameArray.join("."), {}]);
            getFullyQualifiedProperties(val, fullNameArray, acc);
        } else {
            if (val && typeof val.value !== "undefined") {
                acc.push([fullNameArray.join("."), val.value, val]);
            } else {
                acc.push([fullNameArray.join("."), val]);
            }
        }
    }
    return acc;
};

exports["default"] = { tryRead: tryRead, getReader: getReader, getValueSetter: getValueSetter, commandLineSetter: commandLineSetter, getFullyQualifiedProperties: getFullyQualifiedProperties };
module.exports = exports["default"];